/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com> 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using System.Text.RegularExpressions;
using System.IO;
using System.Collections;

namespace MobiServer.Devices.IR.Control
{
    public partial class Base : MobiServer.Devices.ControlBase
    {
        #region Constants/Variables
        string category = "";
        string product = "";
        Hashtable irDataHolder = new Hashtable(); 

        public Data.IRReceiveHandler irReceiveHandler;
        public Data.IRSendHandler irSendHandler;

        public const string DATA_DIR = @"irdata";
        public const string DATA_FILE_HEADER = "Command";
        public const string SETTING_COMMAND_HEADER = "setting_command";
        const string DEFAULT_ITEM_COMMAND = "[Command]";

        bool commandSendFlag = true;
        bool focusFlag = false;

        const string STR_ADD_COMMAND = "R}h: {0}ǉ܂D";
        const string STR_DELETE_COMMAND = "R}h: {0}폜܂D";

        #endregion

        #region Constructor/Destructor
        public Base()
        {
            InitializeComponent();
        }

        public Base(string category, string product, int port)
            : base(Data.Base.Header, GetLabel(category, product))
        {
            InitializeComponent();
            this.category = category;
            this.product = product;
            numericUpDownPort.Value = port;
            LoadIRDataAll();
            LoadSetting();
        }
        #endregion

        #region Properties
        public string Category
        {
            get { return category; }
        }

        public string Product
        {
            get { return product; }
        }
        #endregion
     
        #region Server In/Out
        override public void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                if (CheckAddress(e.Args))
                {
                    Data.Base b = Data.Base.Parse(e.Args);

                    if (b != null)
                    {

                        if (b.State == Data.States.Dettach)
                        {
                            Dettach(true);
                        }
                        else if (b.State == Data.States.Out)
                        {
                            if (irDataHolder.Contains(b.Command))
                            {
                                comboBoxCommand.SelectedItem = b.Command;
                                if (b.Port >= numericUpDownPort.Minimum && b.Port <= numericUpDownPort.Maximum)
                                    numericUpDownPort.Value = b.Port;

                                string command = GetCurrentCommand();
                                
                                if (command != null)
                                {
                                    Data.Base irData = (Data.Base)irDataHolder[command];
                                    irData.State = Data.States.Out;
                                    SendIRData(irData);
                                }
                            }
                        }
                        else if (b.State == Data.States.Set)
                        {
                            if (irDataHolder.Contains(b.Command))
                            {
                                comboBoxCommand.SelectedItem = b.Command;

                                string command = GetCurrentCommand();

                                if (command != null)
                                {
                                    Data.Base irData = (Data.Base)irDataHolder[command];
                                    irData.State = Data.States.Set;
                                    ReceiveIRData(irData);
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }

        void CreateServerWriteEvent(string state)
        {
            //bool flag = false;
            switch (state)
            {
                case Data.States.Attach:
                case Data.States.Dettach:
                    Data.Base b = new Data.Base(state, this.category, this.product);
                    string[] args = b.ToArray();
                    this.CreateServerWriteEvent(args);
                    break;
            }
        }
        #endregion

        #region IR In/Out
        void ReceiveIRData(IR.Data.Base irData)
        {
            Data.IREventArgs e = new Data.IREventArgs(irData);
            if (irReceiveHandler != null)
            {
                irReceiveHandler.Invoke(this, e);
            }
        }

        public bool SetData(IR.Data.Base irData)
        {
            if (irData.Category != this.category || irData.Product != this.product ||
                !irDataHolder.Contains(irData.Command))
            {
                return false;
            }
            else
            {
                irDataHolder[irData.Command] = irData;
                SaveIRData(irData);
                return true;
            }            
        }

        void SendIRData(Data.Base irData)
        {
            Data.IREventArgs e = new Data.IREventArgs(irData);
            if (irSendHandler != null)
            {
                irSendHandler.Invoke(this, e);
            }
        }

        private void buttonSend_Click(object sender, EventArgs e)
        {
            string command = GetCurrentCommand();

            if (command != null)
            {

                Data.Base irData = (Data.Base)irDataHolder[command];
                irData.Port = (int)numericUpDownPort.Value;
                SendIRData(irData);
            }
        }

        private void buttonReceive_Click(object sender, EventArgs e)
        {
            string command = GetCurrentCommand();

            if (command != null)
            {
                Data.Base irData = (Data.Base)irDataHolder[command];
                ReceiveIRData(irData);
            }
        }
        #endregion

        #region Attach/Dettach

        private void buttonDettach_Click(object sender, EventArgs e)
        {
            Dettach(this.groupBox1.Enabled);
            if (groupBox1.Enabled)
            {
                CreateServerWriteEvent(Data.States.Attach);
            }
            else
            {
                CreateServerWriteEvent(Data.States.Dettach);
            }
        }

        public void Dettach(bool flag)
        {
            if (flag)
            {
                buttonDettach.Text = "^";
                this.groupBox1.Enabled = false;
                this.SaveSetting();
            }
            else
            {
                this.groupBox1.Enabled = true;
                buttonDettach.Text = "x";
            }

        }

        #endregion

        #region Get/Add/Delete Command

        string GetCurrentCommand()
        {
            if (comboBoxCommand.SelectedItem == null)
                return null ;

            string command = comboBoxCommand.SelectedItem.ToString();

            if (!irDataHolder.Contains(command))
                return null;
            else
                return command;
        }

        bool AddCommand(string text)
        {
            if (text != "" && !irDataHolder.Contains(text))
            {
                Data.Base irData = new Data.Base(this.category, this.product, text);
                comboBoxCommand.Items.Add(text);
                irDataHolder.Add(text, irData);
                comboBoxCommand.SelectedItem = text;
                SaveIRData(irData);
                PrintMessage(STR_ADD_COMMAND, text);
                return true;
            }
            else
            {
                return false;
            }
        }

        bool DeleteCommand(string text)
        {
            if (text != "" && irDataHolder.Contains(text))
            {
                irDataHolder.Remove(text);
                comboBoxCommand.Items.Remove(text);
                comboBoxCommand.SelectedIndex = 0;
                DeleteCommandFile(text);
                Data.Base irData = new Data.Base(this.category, this.product, text);
                comboBoxCommand.Items.Add(text);
                irDataHolder.Add(text, irData);
                PrintMessage(STR_DELETE_COMMAND, text);
                return true;
            }
            else
            {
                return false;
            }
        }

        bool DeleteCommandFile(string command)
        {
          
            string directory = GetDataDirectory();
            string file = String.Format(@"{0}\{1}_{2}.dat", directory, DATA_FILE_HEADER, command);
            if (File.Exists(file))
            {
                File.Delete(file);
                return true;

            }
            else
            {
                return false;
            }
        }

        private void comboBoxCommand_KeyDown(object sender, KeyEventArgs e)
        {
            string command = comboBoxCommand.Text;

            if (command == DEFAULT_ITEM_COMMAND)
                return;

            switch (e.KeyCode)
            {
                case Keys.Enter:
                    if (irDataHolder.Contains(command))
                    {
                        Data.Base irData = (Data.Base)irDataHolder[command];
                        irData.Port = (int)numericUpDownPort.Value;
                        if (irData.IRBytes == null)
                        {
                            ReceiveIRData(irData);

                        }
                        else
                        {
                            SendIRData(irData);
                        }
                    }
                    else
                    {
                        AddCommand(command);
                    }
                    break;
                case Keys.Delete:
                    DeleteCommand(comboBoxCommand.Text);
                    break;

            }
        }

        #endregion

        #region Save / Load
        string GetDataDirectory()
        {
            string str = DATA_DIR + @"\" + String.Format("{0}_{1}", category, product);
            return str;
        }

        string GetSettingFile()
        {
            string str = SETTING_DIR + @"\" + String.Format("{0}_{1}_{2}.txt", SETTING_COMMAND_HEADER, category, product);
            return str;
        }

        bool LoadIRDataAll()
        {
            string directory = GetDataDirectory();

            if (!Directory.Exists(directory))
                return false;

            string[] files = Directory.GetFiles(directory);

            foreach (string file in files)
            {
                string f = Path.GetFileNameWithoutExtension(file);
                Match m = Regex.Match(f, String.Format("^{0}_(.+)", DATA_FILE_HEADER));
                if (m.Success)
                {
                    string command = m.Groups[1].ToString();

                    byte[] data = null;

                    try
                    {
                        data = File.ReadAllBytes(file);
                        if (data.Length != Data.Base.DATA_LENGTH)
                            data = null;
                    }
                    catch (Exception ex)
                    {
                    }

                    Data.Base b = new Data.Base(Data.States.Out, category, product, command, (int)numericUpDownPort.Value, data);
                    if (irDataHolder.Contains(command))
                    {
                        irDataHolder[command] = b;
                    }
                    else
                    {
                        irDataHolder.Add(command, b);
                        comboBoxCommand.Items.Add(command);
                    }
                }
            }
            return true;
        }

        bool SaveIRData(Data.Base irData)
        {
            string directory = GetDataDirectory();

            if (!Directory.Exists(directory))
                Directory.CreateDirectory(directory);

            string file = String.Format(@"{0}\{1}_{2}.dat", directory, DATA_FILE_HEADER, irData.Command);

            if (irData.IRBytes != null)
            {
                File.WriteAllBytes(file, irData.IRBytes);
            }
            else
            {
                File.WriteAllBytes(file, new byte[1] { (byte)0x00 });

            }
            return true;
        }

        public static Hashtable LoadIRControlAll()
        {
            Hashtable holder = new Hashtable();

            string[] files = Directory.GetFiles(SETTING_DIR);

            foreach (string file in files)
            {
                string f = Path.GetFileNameWithoutExtension(file);
                Match m = Regex.Match(f, String.Format("^{0}_(.+)_(.+)", SETTING_COMMAND_HEADER));
                if (m.Success)
                {
                    string category = m.Groups[1].ToString();
                    string product = m.Groups[2].ToString();

                    Base control = new Base(category, product, 1);

                    if (control.GetEnabled())
                        holder.Add(holder.Count, control);
                }
            }

            return holder;
        }

        public override void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();
            try
            {
                fileStream = new StreamWriter(file, false);
                fileStream.WriteLine("{0}", this.GetEnabled());
                fileStream.WriteLine("{0}", numericUpDownPort.Value);
            }
            catch (Exception ex)
            {
                PrintMessage(ex.Message);
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        public override void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();
            string line = "";
            string[] args = null;

            try
            {
                if (!File.Exists(file))
                    return;

                fileStream = new StreamReader(file);
                line = fileStream.ReadLine();
                this.groupBox1.Enabled = bool.Parse(line);
                line = fileStream.ReadLine();
                numericUpDownPort.Value = decimal.Parse(line);

            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        #endregion

        #region Utility/Others

        public bool Start()
        {
            return false;
        }

        public void Stop()
        {
        }

        static string GetLabel(string category, string product)
        {
            string str = String.Format("{0}:{1}", category, product);
            return str;
        }

        public bool CheckAddress(string category, string product)
        {
            if (Regex.Match(category, this.category, RegexOptions.IgnoreCase).Success &&
                Regex.Match(product, this.product, RegexOptions.IgnoreCase).Success)
                return true;
            else
                return false;
        }

        bool CheckAddress(string[] args)
        {
            string header = args[0];
            string category = args[2];
            string product = args[3];

            if (header == Data.Base.Header && CheckAddress(category, product))
                return true;
            else
                return false;
        }

        public bool GetEnabled()
        {
            return this.groupBox1.Enabled;
        }

        public void SetEnabled(bool enabled)
        {
            this.groupBox1.Enabled = enabled;

            if (enabled && focusFlag)
            {
                this.comboBoxCommand.Focus();
            }

        }

        private void Base_MouseEnter(object sender, EventArgs e)
        {
        }

        private void Base_MouseLeave(object sender, EventArgs e)
        {
            focusFlag = false;
        }

        private void comboBoxCommand_MouseEnter(object sender, EventArgs e)
        {
            focusFlag = true;
            comboBoxCommand.Focus();
        }
        #endregion


    }
}
