/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com> 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace MobiServer.Devices.IR.Data
{
    public class Commands{
        public const byte READ = 0x72;          //r
        public const byte TRANSMIT = 0x74;      //t
        public const byte ACK = 0x59;           //Y

        public const byte START_DATA = 0x53;    //S
        public const byte END_DATA = 0x45;      //E

        public const byte PORT_1 = 0x31;        //1
        public const byte PORT_2 = 0x32;        //2
        public const byte PORT_3 = 0x33;        //3
        public const byte PORT_4 = 0x34;        //4
    }

    public class States
    {
        public const string In = "In";
        public const string Out = "Out";
        public const string Set = "Set";
        public const string Attach = "Attach";
        public const string Dettach = "Dettach";
    }

    public class Base
    {
        public const string Header = "IR";
        public const int DATA_LENGTH = 240;
        string state;
        string category;
        string product;
        string command;
        byte[] bytes;
        int port;
        
        public Base(string state, string category, string product, string command,int port, byte[] data)
        {
            this.state = state;
            this.category = category;
            this.product = product;
            this.command = command;
            this.port = port;
            this.bytes = data;
        }

        public Base(string state, string category, string product, string command, int port)
        {
            this.state = state;
            this.category = category;
            this.product = product;
            this.command = command;
            this.bytes = null;
            this.port = port;
        }

        public Base(string state, string category, string product, string command)
        {
            this.state = state;
            this.category = category;
            this.product = product;
            this.command = command;
            this.bytes = null;
            this.port = 1;  // IR.Data.Commands.PORT_1;
        }

        public Base(string state, string category, string product)
        {
            this.state = state;
            this.category = category;
            this.product = product;
            this.command = "";
            this.bytes = null;
            this.port = 1;  //IR.Data.Commands.PORT_1;
        }

        public string State
        {
            get { return this.state; }
            set { this.state = value; }
        }

        public string Category
        {
            get { return this.category; }
        }

        public string Product
        {
            get { return this.product; }
        }

        public string Command
        {
            get { return this.command; }
        }

        public byte[] IRBytes
        {
            get { return this.bytes; }
            set { this.bytes = value; }
        }

        public int Port
        {
            get { return this.port; }
            set { this.port = value; }
        }

        public string[] ToArray()
        {
            string[] args = null;

            switch (State)
            {
                case States.Attach:
                case States.Dettach:
                    args = new string[4] { Header, State, Category, Product};
                    break;
                case States.In:
                case States.Out:
                    args = new string[5] { Header, State, Category, Product, Command};
                    break;
            }
            return args;
        }

        static public Base Parse(string [] args)
        {
            try
            {
                if (args[0] != Header)
                    return null;

                Base b = null;

                string state = args[1];
                string category = args[2];
                string product = args[3];
 
                if (args.Length < 5)
                {
                    b = new Base(state, category, product);
                }
                else if (args.Length < 6)
                {
                    string command = args[4];
                    b = new Base(state, category, product, command);
                }
                else
                {
                    string command = args[4];
                    int port = int.Parse(args[5]);
                    b = new Base(state, category, product, command, port);
                }
                return b;
            }
            catch (Exception ex)
            {
                return null;
            }

        }
    }
}

