﻿/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com> 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
//using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Threading;
using System.Net.Sockets;

namespace MobiServer
{

    public partial class MainForm : System.Windows.Forms.Form
    {
        int panelIRHeight = 0;
        MobiServer.MessageHandler messageHandler;

        public MainForm()
        {
            InitializeComponent();
            InitPanelIR();
            System.Windows.Forms.Control.CheckForIllegalCrossThreadCalls = false;

            messageHandler += new MobiServer.MessageHandler(messageControl.OnMessageEvent);
            panelIRHeight = panelIR.Height;
        }

        void InitPanelIR()
        {
            // panelIR
            this.panelIR = new MobiServer.Devices.IR.Control.Panel(serverControl, messageControl);
            this.panelIR.Location = new System.Drawing.Point(1, 114);
            this.panelIR.Name = "panelIR";
            this.panelIR.Size = new System.Drawing.Size(655, 122);
            this.panelIR.TabIndex = 2;
            this.panelIR.Resize += new System.EventHandler(this.panelIR_Resize);
            this.Controls.Add(this.panelIR);
        }


        private void panelIR_Resize(object sender, EventArgs e)
        {
            this.Height += panelIR.Height - panelIRHeight;
            panelIRHeight = panelIR.Height;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {

        }

        #region Message Events

        public void PrintError(string m, params object[] args)
        {
            MobiServer.MessageEventArgs e = new MobiServer.MessageEventArgs(String.Format(m, args), MobiServer.States.Error);

            if (messageHandler != null)
                messageHandler(this, e);
        }

        public void PrintMessage(string m, params object[] args)
        {
            MobiServer.MessageEventArgs e = new MobiServer.MessageEventArgs(String.Format(m, args));

            if (messageHandler != null)
                messageHandler(this, e);

            return;
        }

        public void PrintDebug(string m, params object[] args)
        {
            MobiServer.MessageEventArgs e = new MobiServer.MessageEventArgs(String.Format(m, args), MobiServer.States.Debug);

            if (messageHandler != null)
                messageHandler(this, e);
        }

        #endregion

    }
}
